FUNCTION autoz_aaomega_tile, tilename, dir=dir, verbose=verbose, $
  ext1=ext1, header1=header1, ext2=ext2, header2=header2, ext3=ext3, $
  header3=header3, st_lambda=st_lambda, end_lambda=end_lambda, $
  minval=minval, maxval=maxval, clipvalue=clipvalue, os1=os1, os2=os2, $
  template_numbers=template_numbers, dataall=dataall, fits=fits, ascii=ascii
;** Compute redshifts for a reduced AAOmega tile - output of 2dfdr. 
;** The code is known as Autoz, used for the GAMA survey. 
;** Reference: Baldry et al. 2014, MNRAS, 441, 2440
;** tilename should be a string not including .fits extension. 
;** 
;** Keywords: ext1, header1, ext2, etc. to return data from the tile. 
;** st_lambda, end_lambda for maximum range of initial fitting for the 
;** high-pass filtering. Pixels outside wavelength range st_lambda+os1 and 
;** end_lambda-os1 are set to zero in the high-pass filtered (HPF) spectra, 
;** os1=10 is default set in process_spectrum.pro. minval, maxval are the 
;** minimum and maximum pixel values that are assumed to be bad pixels and 
;** therefore set to zero in the HPF spectra. template_numbers can be set 
;** to an integer vector to select the templates used in the cross correlation. 
;** dataall returns the outputs of autoz_singlespec. 
;** 
;** Written by Ivan Baldry. i.baldry@ljmu.ac.uk .

;** Set default directory. 
if n_elements(dir) EQ 0 then dir = 'data/'
file = tilename + '.fits'

;** Lambda scale to use - 3.3 to 4.0 - oversample by 5 from SDSS binning. 
set_up_log_lambda, gap, log_lambda_rebin, velocity_gap=velocity_gap

;** Read in template data and bin onto oversampled scale. 
tdata_rebin = rebin_template_data(log_lambda_rebin, template_numbers, $
  num_templates=num_templates)

;** Read in first three extensions (0-2 in IDL): spectra, variances, table.
print, '% AUTOZ_AAOMEGA_TILE: Reading ', dir+file
ext1 = mrdfits(dir+file, 0, header1)
ext2 = mrdfits(dir+file, 1, header2, /silent)
ext3 = mrdfits(dir+file, 2, header3, /silent)

;** Extract date and location information. 
MJD = sxpar(header1, 'UTMJD')  &  JD = double(MJD) + 2400000.5D
longitude = sxpar(header1, 'LONG_OBS')
latitude  = sxpar(header1, 'LAT_OBS')
altitude  = sxpar(header1, 'ALT_OBS')
print, '% AUTOZ_AAOMEGA_TILE: Observed (MJD,location):', mjd, $
  longitude, latitude, altitude, format='(a,f12.5,f7.1,f7.1,i7)'

;** Wavelength scale.
num_lambda = n_elements(ext1[*,0])
lambda = sxpar(header1,'CRVAL1') + sxpar(header1,'CDELT1') * $
  ( dindgen(num_lambda) + 1 - sxpar(header1,'CRPIX1') )
print, '% AUTOZ_AAOMEGA_TILE: Wavelengths:', lambda[0], lambda[num_lambda-1]

;** Set approximate relative flux 'calibration' as a function of microns.
;** Was obtained usng GAMA standard stars. 
res = [  -3.406,  22.175, -49.208,  55.037, -23.441]
inv_correction = poly_calc(res, lambda/10000.)

;** Select target fibres only. 
use_fibres = where(ext3.type EQ 'P', count)
print, '% AUTOZ_AAOMEGA_TILE: Selected', count, ' fibres of type P'

dataall = set_up_allz_structure(count, num_peaks=num_peaks, /default)

if keyword_set(verbose) then print, 'SPECID, REDSHIFT, CC_SIGMA, TEMPLATE'
for i=0, count-1 do begin
  u_i = use_fibres[i]
;** Get RA and DEC and convert from radians to degrees, for
;**   heliocentric correction. 
  ra = ext3[u_i].ra*!radeg  &  dec = ext3[u_i].dec*!radeg
  helio_velocity = heliocentric(ra, dec, 2000, $
    JD=JD, longitude=longitude, latitude=latitude, altitude=altitude)

;** Calculate the Autoz redshifts, etc. 
  dataall[i] = autoz_singlespec(ext1[*,u_i], sqrt(ext2[*,u_i]), lambda,$
    log_lambda_rebin=log_lambda_rebin, gap=gap, tdata_rebin=tdata_rebin,$
    helio_velocity=helio_velocity, num_peaks=num_peaks,$
    st_lambda=st_lambda, end_lambda=end_lambda, minval=minval, maxval=maxval,$
    clipvalue=clipvalue, os1=os1, os2=os2, inv_correction=inv_correction)

;** Add in target information.
  dataall[i].ra  = ra 
  dataall[i].dec = dec
  dataall[i].specid  = tilename+'_'+string(u_i+1,format='(i3.3)')
  dataall[i].imageid = strcompress(ext3[u_i].name, /remove_all)
  if keyword_set(verbose) then print, dataall[i].specid,$
    dataall[i].z_fitted[0], dataall[i].cc_sigma[0], dataall[i].template[0]
endfor

autoz = convert_z_format(dataall)
!p.multi=[0,1,1]
plot, autoz.z, autoz.cc_fom, psym=6, xr=[-0.01,0.91], xstyle=1, ystyle=16, $
  xtitle='Redshift', ytitle='CC_FOM', title=tilename
oplot, [-1,10], [4.5,4.5], lines=2
oplot, [-1,10], [5.0,5.0], lines=1
print, '% AUTOZ_AAOMEGA_TILE: Number CC_FOM>4.5 =', $
  totalr(autoz.cc_fom GT 4.5), ' out of', n_elements(autoz)

;** output file options
if keyword_set(fits) then mwrfits, autoz, tilename+'_autoz.fits', /create
if keyword_set(ascii) then begin
  openw,1,tilename+'_autoz.txt'
  printf,1,'#',tag_names(autoz),format='(a,20(a,x))'
  for i=0, n_elements(autoz)-1 do printf,1, autoz[i],$
    format='(a,x,2(f9.5,x),a,x,3(i4,x),3(f8.5,x),i2,x,2(f8.5,x),i2,x,5(f8.5,x))'
  close,1
endif

return, autoz

end
