FUNCTION autoz_singlespec, input_spec, input_spec_error, lambda, $
  log_lambda_rebin=log_lambda_rebin, gap=gap, tdata_rebin=tdata_rebin, $
  helio_velocity=helio_velocity, $
  num_peaks=num_peaks, ccinfo=ccinfo, peaks=peaks, highz=highz, $
  st_lambda=st_lambda, end_lambda=end_lambda, minval=minval, maxval=maxval, $
  clipvalue=clipvalue, os1=os1, os2=os2, inv_correction=inv_correction
;** Process spectrum, calculate redshifts and best fit templates using
;** cross-correlation. The code is known as Autoz, used for the GAMA survey. 
;** Reference: Baldry et al. 2014, MNRAS, 441, 2440
;** 
;** Inputs are the spectrum, error spectrum, and air wavelength scale. 
;** The templates are on a vacuum wavelength scale. Nan in the spectrum
;** is assumed to represent bad pixels. 
;**
;** Essential keywords: log_lambda_rebin is the vacuum wavelength scale to 
;** rebin to. tdata_rebin is a structure containing the filtered templates. 
;** helio_velocity is the heliocentric velocity for the observation to 
;** correct for: set to zero if obs. is on heliocentic wavelength scale. 
;** (gap is re-obtained from log_lambda_rebin)
;** 
;** num_peaks is the number of cross-correlation peaks to calculate, the 
;** default is four. inv_correction is the relative flux calibration.
;** ccinfo, peaks return these structures to the calling routine. 
;** highz is set if a larger cross-correlation range is needed. 
;** st_lambda, end_lambda for maximum range of initial fitting for the 
;** high-pass filtering. Pixels outside wavelength range st_lambda+os1 and 
;** end_lambda-os1 are set to zero in the high-pass filtered (HPF) spectra, 
;** os1=10 is default set in process_spectrum.pro. minval, maxval are the 
;** minimum and maximum pixel values for good data, values outside these 
;** are assumed to be bad pixels and therefore set to zero in the HPF spectra. 
;** 
;** Written by Ivan Baldry. 

;** Use single structure for results.
results = set_up_allz_structure(1, num_peaks=num_peaks)

specfilt = process_spectrum(input_spec, input_spec_error, lambda,$
  st_lambda=st_lambda, end_lambda=end_lambda, minval=minval, maxval=maxval,$
  clipvalue=clipvalue, os1=os1, os2=os2, inv_correction=inv_correction,$
  /silent, count_badpixel=count_badpixel)

results.count_badpixel = count_badpixel
results.count_highval  = totalr(specfilt GT 20.)
results.count_lowval   = totalr(specfilt LT -20.)
results.meanad_norm    = mean(abs(specfilt))
results.rms_norm       = rms(specfilt)

;** Convert spectral lambda to vacuum wavelength from air wavelength input.
log_v_lambda = alog10(vacuum_from_air(lambda))  &  N = n_elements(log_v_lambda)
;** Rebin filtered spectrum ensuring zero outside range (zero padding).
spec_rebin = interpol([0., 0., specfilt, 0., 0.], $
   [3.0, log_v_lambda[0]-0.001, log_v_lambda, log_v_lambda[N-1]+0.001, 4.5], $
   log_lambda_rebin)

;** Re-obtain gap from log_lambda_rebin so that it does not have to be set
;** log_lambda_rebin needs to match tdata_rebin.spec in size
;** and be evenly spaced in log wavelength. 
gap = (log_lambda_rebin[1000] - log_lambda_rebin[0])/1000.0D

;** Do all cross correlations and find the highest peaks.
ccinfo = do_crosscorr(spec_rebin, tdata_rebin,$
  helio_vel=helio_velocity, gap=gap, highz=highz)
peaks = find_highest_peaks(ccinfo)

;** Note *peaks.template* refers to the *index* within ccinfo and not the 
;** template number. 
results.helio_vc = helio_velocity
results.template = ccinfo[peaks.template].template_num
results.cc_sigma = peaks.crosscorr
results.z_peak   = peaks.redshift
results.num_tp   = ccinfo[peaks.template].num_turningpoints

for p_i = 0, num_peaks-1 do begin
  cci = ccinfo[peaks[p_i].template]  &  best_pt = peaks[p_i].shift_index
;** Find fitted redshift using quadratic.
  results.z_fitted[p_i] = find_max(cci.shifts[best_pt-3:best_pt+3], $
    cci.crosscorr_raw[best_pt-3:best_pt+3], n=2, status=status, /double) 
  results.cc_raw[p_i] = cci.crosscorr_raw[best_pt]
  halfpeak = 0.5 * cci.crosscorr_raw[best_pt]
  results.halfpeak_pixels[p_i] = $
    totalr(cci.crosscorr_raw[best_pt-43:best_pt+43] GT halfpeak)
endfor

return, results

end
