FUNCTION convert_z_format, allz
;** Convert format of autoz files, compute figure of merit. The output of 
;** autoz_singlespec uses a structure with arrays embedded. This is not 
;** standard for SQL databases. This function converts the first output to 
;** a structure with only standard columns. The figure of merit (FOM) is 
;** determined by this routine using the GAMA calibration. 
;** 
;** Written by Ivan Baldry. 
;** Reference: Baldry et al. 2014, MNRAS, 441, 2440

num = n_elements(allz)
autoz = replicate({autoz, specid: '', ra: 0.0D, dec: 0.0D, imageid: '',$
  count_badpixel: 0, count_highval: 0, count_lowval: 0, rms_mad_ratio: 0.0,$
  z: 0.0, cc_sigma: 0.0, template: 0, z2: 0.0, cc_sigma2: 0.0, template2: 0,$
  cc_sigma3: 0., cc_sigma4: 0., cc_sigma1to234: 0.0, $
  cc_fom: 0.0, prob: 0.0, vel_err: 0.0}, num)

;** Copy information between structures - including change of format.
struct_assign, allz, autoz
autoz.z         = allz.z_fitted[0]
autoz.z2        = allz.z_fitted[1]
autoz.template  = allz.template[0]
autoz.template2 = allz.template[1]
autoz.cc_sigma  = allz.cc_sigma[0]
autoz.cc_sigma2 = allz.cc_sigma[1]
autoz.cc_sigma3 = allz.cc_sigma[2]
autoz.cc_sigma4 = allz.cc_sigma[3]

;** Calculate root mean square to mean absolute deviation ratio. 
autoz.rms_mad_ratio  = allz.rms_norm / allz.meanad_norm

;** Calculate ratio of first peak compared to RMS of 2nd/3rd/4th. 
autoz.cc_sigma1to234 = allz.cc_sigma[0] / $
  sqrt( total(allz.cc_sigma[1:3]^2, 1)/3. )

;** The following calibrations were determined for the GAMA survey. 
;** It is unclear how they will perform with different surveys. 
;** See paper Baldry et al. 2014 for details. 
;** This does not include the redshift penalty function. 

;** Define FOM, lower of cc_sigma and a converted cc_sigma1to234.
autoz.cc_fom = autoz.cc_sigma < poly_calc([0.4, 2.8], autoz.cc_sigma1to234)

;** Adjustment to FOM for large rms_mad_ratio. 
rmr_adjustment = (1.5 * (autoz.rms_mad_ratio - 1.8)) > 0.0 < 2.1
autoz.cc_fom = (autoz.cc_fom - rmr_adjustment) > 2.6

;** Set redshift confidence from CC_FOM. 
autoz.prob = tanh_0to1(double(autoz.cc_fom), [3.70, 0.70]) 

;** computation of velocity errors, GAMA calibration. 
;** velocity fwhm - set between 25 and 80 rebinned pixels
vel_fwhm = (allz.halfpeak_pixels[0] * 13.8) > 345. < 1104.
vel_to_sig = vel_fwhm / (1. + autoz.cc_sigma)
autoz.vel_err = sqrt( 19.12^2 + (0.525*vel_to_sig)^2 )

return, autoz

end
