FUNCTION do_crosscorr, spec_rebin, tdata_rebin, gap_log10=gap_log10,$
  helio_vel=helio_vel, highz=highz
;** Compute the cross correlations and save information to structure.
;** Reference: Baldry et al. 2014, MNRAS, 441, 2440
;** Written by Ivan Baldry. 

num_templates = n_elements(tdata_rebin)
include_cc = bytarr(num_templates) + 1B

if keyword_set(highz) then num_corr = 28000L else num_corr = 14000L

;** Set up output structure.
num_pts_save = 2*num_corr+1
dataout = replicate({ccinfo_v3, template_num: -1, shifts: fltarr(num_pts_save),$
  crosscorr_raw: fltarr(num_pts_save), crosscorr: fltarr(num_pts_save),$
  mask_info: bytarr(num_pts_save), num_turningpoints: 0}, num_templates)
dataout.template_num = tdata_rebin.template_num

for i=0, num_templates-1 do begin

  t_num = tdata_rebin[i].template_num
  case 1 of
;** late-type stellar templates - power is at red end
    totalr(t_num EQ [11,12,13,14,15,17,19,22]): begin
      rms_z_range = [-0.2,0.4]  &  allowed_z_range = [-0.002, 0.002] 
    end
;** remaining stellar templates
    t_num LE 22: begin
      rms_z_range = [-0.1,0.5]  &  allowed_z_range = [-0.002, 0.002] 
    end
;** original galaxy templates
    t_num GE 23 and t_num LE 28: begin
      rms_z_range = [-0.1,0.8]  &  allowed_z_range = [-0.005, 0.800]
    end
;** QSO templates - not working reliably with GAMA - needs highz set
    t_num GE 29 and t_num LE 32: begin
      rms_z_range = [-0.1,2.5]  &  allowed_z_range = [ 0.350, 2.500]
    end
;** other galaxy templates
    t_num GE 33: begin
      rms_z_range = [-0.1,0.9]  &  allowed_z_range = [-0.005, 0.900]
    end
  endcase

;** determine raw cross correlation
  crosscorr_raw = cross_corr(tdata_rebin[i].spec, spec_rebin, num_corr)

;** lambda shift * (1+z) of template * (1+z) of heliocorrection
  shifts = 10.^((dindgen(2*num_corr+1)-num_corr)*gap_log10) * $
    (1. + tdata_rebin[i].redshift) * (1. + helio_vel/2.998E5) - 1.0

;** set criteria
  criteria_search  = criteria_between(shifts, allowed_z_range)
  criteria_rms     = criteria_between(shifts, rms_z_range)
  criteria_pospeak = criteria_peak(crosscorr_raw)  ;** positive peaks
  criteria_negpeak = criteria_peak(-crosscorr_raw) ;** negative peaks
  criteria_tp      = criteria_pospeak or criteria_negpeak

;** Subtract trimmed mean excluding top and bottom 4% of points.
;** This brings more symmetry to positive and negative peaks.
  use_rms = where(criteria_rms, count_rms)
  crosscorr = crosscorr_raw - mean_reject(crosscorr_raw[use_rms], count_rms/25)

;** normalisation using turning points - divide by root mean square.
  use_norm = where(criteria_rms and criteria_tp, num_turningpoints)
  crosscorr = crosscorr / rms(crosscorr[use_norm])

;** Alternative normalization using values of positive peaks and 
;**   negative values of negative peaks.
;** I found this did not work as well. 
;  usepos = where(criteria_rms and criteria_pospeak, c_pos)
;  useneg = where(criteria_rms and criteria_negpeak, c_neg)
;  num_turningpoints = c_pos + c_neg
;  test_vals = [crosscorr[usepos], -crosscorr[useneg]]
;  trimmed_mean = mean_reject(test_vals, num_turningpoints/25)
;  sd_estimate = rms(test_vals - trimmed_mean)
;  crosscorr = (crosscorr - trimmed_mean) / sd_estimate

;** assign information to structure for output function
  dataout[i].shifts        = shifts
  dataout[i].crosscorr_raw = crosscorr_raw
  dataout[i].crosscorr     = crosscorr
  dataout[i].mask_info = 1*criteria_search + 2*criteria_rms + $
    4*criteria_pospeak + 8*criteria_negpeak + 16*criteria_tp
  dataout[i].num_turningpoints = num_turningpoints

endfor

return, dataout

end
