FUNCTION process_spectrum, input_spec, input_spec_error, lambda,$
  st_lambda=st_lambda, end_lambda=end_lambda, minval=minval, maxval=maxval,$
  clipvalue=clipvalue, os1=os1, os2=os2, inv_correction=inv_correction,$
  silent=silent, spec=spec, intermediate=spec_intermediate, error=spec_error,$
  lowpass=spec_lowpass, use_fit=use_fit, count_badpixel=count_badpixel,$
  baddata_error_value=baddata_error_value, criteria_badspec=criteria_badspec,$
  mean_deviation=mean_deviation
;** Input a spectrum, and create filtered version for cross-correlation.
;** Reference: Baldry et al. 2014, MNRAS, 441, 2440
;** Written by Ivan Baldry. 

;** Copy vectors so that originals do not change.
spec = input_spec  &  spec_error = input_spec_error

;** Set other parameters with defaults if not defined. 
if keyword_set(st_lambda)  EQ 0 then st_lambda  = 3726.
if keyword_set(end_lambda) EQ 0 then end_lambda = 8850.  
if n_elements(os1) EQ 0 then os1 = 10.
if n_elements(os2) EQ 0 then os2 = 60.
if keyword_set(baddata_error_value) EQ 0 then baddata_error_value = 1E10
if n_elements(minval) EQ 0 then minval = -1.0E4  
if n_elements(maxval) EQ 0 then maxval =  1.0E6 
if keyword_set(clipvalue) EQ 0 then clipvalue = 25.

;** Criteria for bad spectral points: non-finite - flagged as bad - 
;**   or large deviations of sky-subtracted data - assumed bad pixels.
criteria_badspec = (finite(spec) EQ 0) or (spec LT minval) or (spec GT maxval)
;** Criteria for bad errors: non-finite or <= zero
criteria_baderror = (finite(spec_error) EQ 0) or (spec_error LE 0.)

;** Apply relative flux calibration.
if keyword_set(inv_correction) then begin
  spec = spec / inv_correction
  spec_error = spec_error / inv_correction
endif

;** Robustly high-pass filter the spectrum.
specfilt = fit_and_filter(lambda, spec, st_lambda, end_lambda, $
  os1=os1, os2=os2, use_fit=use_fit, silent=silent, $
  spec_smooth=spec_smooth, spec_fit=spec_fit)
spec_lowpass = spec_smooth + spec_fit

;** Save intermediate filtered spectrum for checking.
spec_intermediate = specfilt

use_ok_error = where(criteria_baderror EQ 0)
;** Broaden error over all lines
spec_error[use_ok_error] = adjust_broaden(spec_error[use_ok_error])
;** Set minimum error to avoid anomalously low errors
spec_error[use_ok_error] = spec_error[use_ok_error] > $
  (0.7*median_adjust(spec_error[use_ok_error], 13))

;** Set bad-data error value where there exists any bad data .
use_baddata = where(criteria_baderror or criteria_badspec, count_badpixel)
if count_badpixel GE 1 then begin
  spec_error[use_baddata] = baddata_error_value
  specfilt[use_baddata] = 0.0
endif

;** divide by sigma^2 as suggested by Saunders, Cannon and Sutherland
specfilt = specfilt / spec_error^2
;** clip value, was 30. 
specfilt = normalise_meandev(specfilt, clip=clipvalue, use=use_fit,$
  mean_deviation=mean_deviation)

return, specfilt

end
